#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Part of the PsychoPy library
# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
# Distributed under the terms of the GNU General Public License (GPL).

"""utility classes for the Builder
"""

from __future__ import absolute_import, division, print_function

import os
from builtins import object

from wx.lib.agw.aui.aui_constants import *
from wx.lib.agw.aui.aui_utilities import IndentPressedBitmap, ChopText, TakeScreenShot
import sys
import wx
import wx.lib.agw.aui as aui
from wx.lib import platebtn

import psychopy
from psychopy import logging
from . import pavlovia_ui
from . import icons
from .themes import ThemeMixin
from psychopy.localization import _translate

class FileDropTarget(wx.FileDropTarget):
    """On Mac simply setting a handler for the EVT_DROP_FILES isn't enough.
    Need this too.
    """

    def __init__(self, targetFrame):
        wx.FileDropTarget.__init__(self)
        self.target = targetFrame

    def OnDropFiles(self, x, y, filenames):
        logging.debug(
            'PsychoPyBuilder: received dropped files: %s' % filenames)
        for fname in filenames:
            if fname.endswith('.psyexp') or fname.lower().endswith('.py'):
                self.target.fileOpen(filename=fname)
            else:
                logging.warning(
                    'dropped file ignored: did not end in .psyexp or .py')
        return True


class WindowFrozen(object):
    """
    Equivalent to wxWindowUpdateLocker.

    Usage::

        with WindowFrozen(wxControl):
            update multiple things
        # will automatically thaw here

    """

    def __init__(self, ctrl):
        self.ctrl = ctrl

    def __enter__(self):  # started the with... statement
        # Freeze should not be called if platform is win32.
        if sys.platform == 'win32':
            return self.ctrl

        # check it hasn't been deleted
        #
        # Don't use StrictVersion() here, as `wx` doesn't follow the required
        # numbering scheme.
        if self.ctrl is not None and wx.__version__[:3] <= '2.8':
            self.ctrl.Freeze()
        return self.ctrl

    def __exit__(self, exc_type, exc_val, exc_tb):
        # Thaw should not be called if platform is win32.
        if sys.platform == 'win32':
            return
        # check it hasn't been deleted
        if self.ctrl is not None and self.ctrl.IsFrozen():
            self.ctrl.Thaw()


def getSystemFonts(encoding='system', fixedWidthOnly=False):
    """Get a list of installed system fonts.

    Parameters
    ----------
    encoding : str
        Get fonts with matching encodings.
    fixedWidthOnly : bool
        Return on fixed width fonts.

    Returns
    -------
    list
        List of font facenames.

    """
    fontEnum = wx.FontEnumerator()

    encoding = "FONTENCODING_" + encoding.upper()
    if hasattr(wx, encoding):
        encoding = getattr(wx, encoding)

    return fontEnum.GetFacenames(encoding, fixedWidthOnly=fixedWidthOnly)


class PsychopyToolbar(wx.ToolBar, ThemeMixin):
    """Toolbar for the Builder/Coder Frame"""
    def __init__(self, frame):
        wx.ToolBar.__init__(self, frame)
        self.frame = frame
        self.app = self.frame.app
        self._needMakeTools = True
        # Configure toolbar appearance
        self.SetWindowStyle(wx.TB_HORIZONTAL | wx.NO_BORDER | wx.TB_FLAT | wx.TB_NODIVIDER)
        #self.SetBackgroundColour(ThemeMixin.appColors['frame_bg'])
        # Set icon size (16 for win/linux small mode, 32 for everything else
        self.iconSize = 32  # mac: 16 either doesn't work, or looks bad
        self.SetToolBitmapSize((self.iconSize, self.iconSize))
        # OS-dependent tool-tips
        ctrlKey = 'Ctrl+'
        if sys.platform == 'darwin':
            ctrlKey = 'Cmd+'
        # keys are the keyboard keys, not the keys of the dict
        self.keys = {k: self.frame.app.keys[k].replace('Ctrl+', ctrlKey)
                for k in self.frame.app.keys}
        self.keys['none'] = ''
        # self.makeTools()  # will be done when theme is applied
        # Finished setup. Make it happen

    def makeTools(self):
        frame = self.frame
        # Create tools
        cl = frame.__class__.__name__
        pavButtons = pavlovia_ui.toolbar.PavloviaButtons(
                frame, toolbar=self, tbSize=self.iconSize)
        if frame.__class__.__name__ == 'BuilderFrame':
            self.addPsychopyTool(
                    name='filenew',
                    label=_translate('New'),
                    shortcut='new',
                    tooltip=_translate("Create new experiment file"),
                    func=self.frame.app.newBuilderFrame)  # New
            self.addPsychopyTool(
                    name='fileopen',
                    label=_translate('Open'),
                    shortcut='open',
                    tooltip=_translate("Open an existing experiment file"),
                    func=self.frame.fileOpen)  # Open
            self.frame.bldrBtnSave = self.addPsychopyTool(
                        name='filesave',
                        label=_translate('Save'),
                        shortcut='save',
                        tooltip=_translate("Save current experiment file"),
                        func=self.frame.fileSave)  # Save
            self.addPsychopyTool(
                    name='filesaveas',
                    label=_translate('Save As...'),
                    shortcut='saveAs',
                    tooltip=_translate("Save current experiment file as..."),
                    func=self.frame.fileSaveAs)  # SaveAs
            self.frame.bldrBtnUndo = self.addPsychopyTool(
                        name='undo',
                        label=_translate('Undo'),
                        shortcut='undo',
                        tooltip=_translate("Undo last action"),
                        func=self.frame.undo)  # Undo
            self.frame.bldrBtnRedo = self.addPsychopyTool(
                        name='redo',
                        label=_translate('Redo'),
                        shortcut='redo',
                        tooltip=_translate("Redo last action"),
                        func=self.frame.redo)  # Redo
            self.AddSeparator()  # Seperator
            self.addPsychopyTool(
                    name='monitors',
                    label=_translate('Monitor Center'),
                    shortcut='none',
                    tooltip=_translate("Monitor settings and calibration"),
                    func=self.frame.app.openMonitorCenter)  # Monitor Center
            self.addPsychopyTool(
                    name='cogwindow',
                    label=_translate('Experiment Settings'),
                    shortcut='none',
                    tooltip=_translate("Edit experiment settings"),
                    func=self.frame.setExperimentSettings)  # Settings
            self.AddSeparator()
            self.addPsychopyTool(
                    name='compile',
                    label=_translate('Compile Script'),
                    shortcut='compileScript',
                    tooltip=_translate("Compile to script"),
                    func=self.frame.compileScript)  # Compile
            self.frame.bldrBtnRunner = self.addPsychopyTool(
                    name='runner',
                    label=_translate('Runner'),
                    shortcut='runnerScript',
                    tooltip=_translate("Send experiment to Runner"),
                    func=self.frame.runFile)  # Run
            self.frame.bldrBtnRun = self.addPsychopyTool(
                    name='run',
                    label=_translate('Run'),
                    shortcut='runScript',
                    tooltip=_translate("Run experiment"),
                    func=self.frame.runFile)  # Run
            self.AddSeparator()  # Seperator
            pavButtons.addPavloviaTools()
        elif frame.__class__.__name__ == 'CoderFrame':
            self.addPsychopyTool('filenew', _translate('New'), 'new',
                                 _translate("Create new experiment file"),
                                 self.frame.fileNew)  # New
            self.addPsychopyTool('fileopen', _translate('Open'), 'open',
                                 _translate("Open an existing experiment file"),
                                 self.frame.fileOpen)  # Open
            self.frame.cdrBtnSave = \
                self.addPsychopyTool('filesave', _translate('Save'), 'save',
                                     _translate("Save current experiment file"),
                                     self.frame.fileSave)  # Save
            self.addPsychopyTool('filesaveas', _translate('Save As...'), 'saveAs',
                                 _translate("Save current experiment file as..."),
                                 self.frame.fileSaveAs)  # SaveAs
            self.frame.cdrBtnUndo = \
                self.addPsychopyTool('undo', _translate('Undo'), 'undo',
                                     _translate("Undo last action"),
                                     self.frame.undo)  # Undo
            self.frame.cdrBtnRedo = \
                self.addPsychopyTool('redo', _translate('Redo'), 'redo',
                                     _translate("Redo last action"),
                                     self.frame.redo)  # Redo
            self.AddSeparator()  # Seperator
            self.addPsychopyTool('monitors', _translate('Monitor Center'), 'none',
                                 _translate("Monitor settings and calibration"),
                                 self.frame.app.openMonitorCenter)
            self.addPsychopyTool('color', _translate('Color Picker'), 'none',
                                 _translate("Color Picker -> clipboard"),
                                 self.frame.app.colorPicker)
            self.AddSeparator()
            self.frame.cdrBtnRunner = self.addPsychopyTool(
                    'runner', _translate('Runner'), 'runnerScript',
                    _translate("Send experiment to Runner"),
                    self.frame.runFile)
            self.frame.cdrBtnRun = self.addPsychopyTool(
                    'run', _translate('Run'), 'runScript',
                    _translate("Run experiment"),
                    self.frame.runFile)
            self.AddSeparator()
            pavButtons.addPavloviaTools(
                buttons=['pavloviaSync', 'pavloviaSearch', 'pavloviaUser'])
        frame.btnHandles.update(pavButtons.btnHandles)
        self.Realize()

    def addPsychopyTool(self, name, label, shortcut, tooltip, func,
                        emblem=None):
        if not name.endswith('.png'):
            name += '.png'
        item = self.app.iconCache.makeBitmapButton(parent=self, filename=name,
                                                   name=label,
                                                   label=("%s [%s]" % (
                                                       label,
                                                       self.keys[shortcut])),
                                                   emblem=emblem, toolbar=self,
                                                   tip=tooltip,
                                                   size=self.iconSize)
        # Bind function
        self.Bind(wx.EVT_TOOL, func, item)
        return item


class PsychopyPlateBtn(platebtn.PlateButton, ThemeMixin):
    def __init__(self, parent, id=wx.ID_ANY, label='', bmp=None,
                 pos=wx.DefaultPosition, size=wx.DefaultSize,
                 style=1, name=wx.ButtonNameStr):
        platebtn.PlateButton.__init__(self, parent, id, label, bmp, pos, size, style, name)
        self.parent = parent
        self.__InitColors()
        self._applyAppTheme()

    def _applyAppTheme(self):
        cs = ThemeMixin.appColors
        self.__InitColors()
        self.SetBackgroundColour(wx.Colour(self.parent.GetBackgroundColour()))
        self.SetPressColor(cs['txtbutton_bg_hover'])
        self.SetLabelColor(cs['text'],
                           cs['txtbutton_fg_hover'])

    def __InitColors(self):
        cs = ThemeMixin.appColors
        """Initialize the default colors"""
        colors = dict(default=True,
                      hlight=cs['txtbutton_bg_hover'],
                      press=cs['txtbutton_bg_hover'],
                      htxt=cs['text'])
        return colors

class PsychopyScrollbar(wx.ScrollBar):
    def __init__(self, parent, ori=wx.VERTICAL):
        wx.ScrollBar.__init__(self)
        if ori == wx.HORIZONTAL:
            style = wx.SB_HORIZONTAL
        else:
            style = wx.SB_VERTICAL
        self.Create(parent, style=style)
        self.ori = ori
        self.parent = parent
        self.Bind(wx.EVT_SCROLL, self.DoScroll)
        self.Resize()

    def DoScroll(self, event):
        if self.ori == wx.HORIZONTAL:
            w = event.GetPosition()
            h = self.parent.GetScrollPos(wx.VERTICAL)
        elif self.ori == wx.VERTICAL:
            w = self.parent.GetScrollPos(wx.HORIZONTAL)
            h = event.GetPosition()
        else:
            return
        self.parent.Scroll(w, h)
        self.Resize()

    def Resize(self):
        sz = self.parent.GetSize()
        vsz = self.parent.GetVirtualSize()
        start = self.parent.GetViewStart()
        if self.ori == wx.HORIZONTAL:
            sz = (sz.GetWidth(), 20)
            vsz = vsz.GetWidth()
        elif self.ori == wx.VERTICAL:
            sz = (20, sz.GetHeight())
            vsz = vsz.GetHeight()
        self.SetDimensions(start[0], start[1], sz[0], sz[1])
        self.SetScrollbar(
            position=self.GetScrollPos(self.ori),
            thumbSize=10,
            range=1,
            pageSize=vsz
        )


class FrameSwitcher(wx.Menu):
    """Menu for switching between different frames"""
    def __init__(self, parent):
        wx.Menu.__init__(self)
        self.parent = parent
        self.app = parent.app
        self.itemFrames = {}
        # Listen for window switch
        self.next = self.Append(wx.ID_MDI_WINDOW_NEXT,
                                _translate("&Next Window\t%s") % self.app.keys['cycleWindows'],
                                _translate("&Next Window\t%s") % self.app.keys['cycleWindows'])
        self.Bind(wx.EVT_MENU, self.nextWindow, self.next)
        self.AppendSeparator()
        # Add creator options
        self.minItemSpec = [
            {'label': "Builder", 'class': psychopy.app.builder.BuilderFrame, 'method': self.app.showBuilder},
            {'label': "Coder", 'class': psychopy.app.coder.CoderFrame, 'method': self.app.showCoder},
            {'label': "Runner", 'class': psychopy.app.runner.RunnerFrame, 'method': self.app.showRunner},
        ]
        for spec in self.minItemSpec:
            if not isinstance(self.Window, spec['class']):
                item = self.Append(
                    wx.ID_ANY, spec['label'], spec['label']
                )
                self.Bind(wx.EVT_MENU, spec['method'], item)
        self.AppendSeparator()
        self.updateFrames()

    @property
    def frames(self):
        return self.parent.app.getAllFrames()

    def updateFrames(self):
        """Set items according to which windows are open"""
        self.next.Enable(len(self.frames) > 1)
        # Make new items if needed
        for frame in self.frames:
            if frame not in self.itemFrames:
                if frame.filename:
                    label = type(frame).__name__.replace("Frame", "") + ": " + os.path.basename(frame.filename)
                else:
                    label = type(frame).__name__.replace("Frame", "")
                self.itemFrames[frame] = self.AppendRadioItem(wx.ID_ANY, label, label)
                self.Bind(wx.EVT_MENU, self.showFrame, self.itemFrames[frame])
        # Edit items to match frames
        for frame in self.itemFrames:
            item = self.itemFrames[frame]
            if not item:
                continue
            if frame not in self.frames:
                # Disable unused items
                item.Enable(False)
            else:
                # Rename item
                if frame.filename:
                    self.itemFrames[frame].SetItemLabel(
                        type(frame).__name__.replace("Frame", "") + ": " + os.path.basename(frame.filename)
                    )
                else:
                    self.itemFrames[frame].SetItemLabel(
                        type(frame).__name__.replace("Frame", "") + ": None"
                    )
            item.Check(frame == self.Window)
        self.itemFrames = {key: self.itemFrames[key] for key in self.itemFrames if self.itemFrames[key] is not None}

    def showFrame(self, event=None):
        itemFrames = event.EventObject.itemFrames
        frame = [key for key in itemFrames if itemFrames[key].Id == event.Id][0]
        frame.Show(True)
        frame.Raise()
        self.parent.app.SetTopWindow(frame)
        self.updateFrames()

    def nextWindow(self, event=None):
        """Cycle through list of open windows"""
        current = event.EventObject.Window
        i = self.frames.index(current)
        while self.frames[i] == current:
            i -= 1
        self.frames[i].Raise()
        self.frames[i].Show()
        self.updateFrames()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Migration helper script for Coder demos: Do the conversion

  For all demos/coder/subdir/*.py (there are ~70 or so total), write out a new file: coder_updated/subdir/*
"""
from __future__ import absolute_import, print_function

import sys
import glob
import os
import re
import io


valid_var_re = re.compile(r"^[a-zA-Z_][\w]*$")
coding = '# -*- coding: utf-8 -*-\n'
demo_license = '\n# The contents of this file are in the public domain.\n'
division = 'from __future__ import division'


def get_contents(f1):
    with io.open(f1, 'r', encoding='utf-8-sig') as f:
        return f.read()


def reposition_division(f1):
    lines = f1.splitlines()
    if lines[2] == division:
        lines.pop(2)
    found = 0
    for i, line in enumerate(lines):
        if line.startswith('"""'):
            found += 1
            if found == 2:
                if division in '\n'.join(lines):
                    break  # already in the right place
                lines.insert(i + 1, '')
                lines.insert(i + 2, division)
                break
                
    return '\n'.join(lines)


def add_shebang_encoding_future(f1):
    if not f1.startswith('#!'):
        f1 = '#!/usr/bin/env python\n' + f1
    
    if '# -*- coding:' not in f1:
        f = f1.split('\n', 1)
        f1 = f[0] + '\n' + coding + f[1]
    
    if not division in f1:
        f1 = f1.replace(coding, coding + division + '\n\n', 1)
    return f1


def remove_doublesharp_trailing_whitespace(f1):
    f1 = f1.replace('##', '#')
    f1 = f1.replace('\n#\n', '\n\n')
    return '\n'.join([line.rstrip() for line in f1.splitlines()]) + '\n'


def replace_PatchStim(f1):
    return f1.replace('PatchStim', 'GratingStim')


def replace_xrange(f1):
    return f1.replace('xrange', 'range')


def replace_myWin_win(f1):
    f1 = f1.replace('myWin', 'win')
    if 'iohub' in f1:
        f1 = f1.replace('window', 'win')
    return f1.replace('win.update()', 'win.flip')


def add_win_close_quit_demo_license(f1):
    # remove first, then add back consistently
    lines = f1.strip().splitlines()
    # need to avoid removing if they are indented:
    if lines[-2].startswith('win.close()') or lines[-2].startswith('core.quit()'):
        lines[-2] = ''
    if lines[-1].startswith('win.close()') or lines[-1].startswith('core.quit()'):
        lines[-1] = ''
    f1 = '\n'.join(lines)
    
    f1 += '\n'
    if 'Window(' in f1:  #  a visual.Window() is defined somewhere
        f1 += '\nwin.close()\n'
    if [line for line in lines if 'psychopy' in line and 'core' in line and 'import' in line]:
        f1 += 'core.quit()\n'

    if not demo_license in f1:
        f1 = f1 + demo_license
    
    return f1


def convert_inline_comments(f1):
    lines = f1.splitlines()
    for i, line in enumerate(lines):
        lines[i] = line.replace('#', '# ').replace('#  ', '# ')
        if '#' in line and not line.strip().startswith('#'):
            lines[i] = lines[i].replace('#', '  #').replace('   #', '  #').replace('   #', '  #')
    return '\n'.join(lines)


def replace_commas_etc(f1):
    # do after shebang, encoding
    f1 = f1.replace(',', ', ').replace(',  ', ', ')
    f1 = f1.replace('=visual', ' = visual')
    f1 = f1.replace('=data', ' = data').replace('=numpy', ' = numpy')
    for op in ['+', '*', '>', '<', '==']:  # too tricky, don't do: % = / -
        f1 = f1.replace(op, ' ' + op + ' ')
        f1 = f1.replace('  ' + op, ' ' + op).replace(op + '  ', op + ' ')
    f1 = f1.replace('> =', '>= ').replace(' < =', ' <= ')
    f1 = f1.replace('> >', '>> ').replace('< <', '<< ')
    f1 = f1.replace('* *', '**').replace('# !', '#!').replace('- * -', '-*-')
    f1 = f1.replace('+ =', '+= ').replace('+=  ', '+= ').replace('- =', '-= ').replace('* =', '*= ')
    f1 = f1.replace(' + + ', '++')  # bits++, mono++ etc
    
    lines = f1.splitlines()
    for i, line in enumerate(lines):
        if line.strip().startswith('#'):
            continue
        if line.strip().endswith(';'):
            lines[i] = line.strip('; ')
        if ':' in line and not ('"' in line or "'" in line):
            lines[i] = line.replace(':', ': ').replace(': \n', ':\n').replace(':  ', ': ')
    f1 = '\n'.join(lines)
    f1 = f1.replace('\n\n\n', '\n\n')
    f1 = f1.replace('\n\n"""', '\n"""').replace('\n"""','\n\n"""', 1)
    f1 = f1.replace('\n"""', '\n"""\n').replace('\n"""\n\n', '\n"""\n')
    
    return f1


def is_var_name(name):
    return all([valid_var_re.match(n) for n in name.split('.')])


def replace_equals(f1):
    lines = f1.splitlines()
    for i, line in enumerate(lines):
        if line.strip().startswith('#'):
            continue
        if '=' in line:
            left, right = line.split('=', 1)
            if is_var_name(left):
                lines[i] = ' = '.join([left, right])
                lines[i] = lines[i].replace('=  ', '=')
    return '\n'.join(lines)


def uk_to_us_spelling(f1):
    f1 = f1.replace('centre', 'center').replace('centerd', 'centered')
    f1 = f1.replace('nitialise', 'nitialize')
    f1 = f1.replace('colour', 'color')
    f1 = f1.replace('analyse', 'analyze')
    return f1


def split_multiline(f1):
    lines = f1.splitlines()
    for i, line in enumerate(lines):
        if line.strip().startswith('#'):
            continue
        if 'if' in line and ':' in line and not ('"' in line or "'" in line) and not line.strip().endswith(':'):
            pre, post = line.split(':', 1)
            if post and not (post.strip().startswith('#') or ']' in line):
                pre_indent = '    '
                count = 0
                while pre.startswith(pre_indent):
                    count += 1
                    pre_indent = '    ' * count
                lines[i] = pre + ':\n' + pre_indent + post.strip()
    return '\n'.join(lines)


def demo_update_one(filename):
    """convert file contents to updated style etc
    """
    f = get_contents(filename)
    if not len(f.strip()):
        return ''  # eg __init__.py
    f = remove_doublesharp_trailing_whitespace(f)
    f = add_shebang_encoding_future(f)
    f = add_win_close_quit_demo_license(f)
    f = convert_inline_comments(f)
    f = replace_xrange(f)
    f = replace_commas_etc(f)  # do after shebang, encoding
    f = replace_PatchStim(f)
    f = replace_myWin_win(f)
    f = replace_equals(f)
    f = uk_to_us_spelling(f)
    f = split_multiline(f)
    return f


def demo_update_two(filename):
    """repostion division statement
    """
    f = get_contents(filename)
    if not len(f.strip()):
        return ''  # eg __init__.py
    #f = reposition_division(f)
    f = f + '\n'
    return f


if __name__ == '__main__':
    # run from within psychopy/demos/
    dirs = [d for d in glob.glob('coder/*') if os.path.isdir(d)]
    if not dirs:
        sys.exit('in wrong directory')
    if not os.path.isdir('coder_updated'):
        os.mkdir('coder_updated')
    for d in dirs:
        if not os.path.isdir(d.replace('coder', 'coder_updated')):
            os.mkdir(d.replace('coder', 'coder_updated'))
        py = glob.glob(d + '/*.py')
        for f1 in py:
            if '__init__.py' in f1:
                continue
            new = demo_update_two(f1)
            
            #"""
            out = f1.replace('coder', 'coder_updated')
            with io.open(out, 'wb') as fh:
                fh.write(new)
            print(new)
            
            try:
                compile(new, '', 'exec')
            except Exception:
                print(out)
                raise



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

import errno
from builtins import object
import os
import sys
import platform
from psychopy.constants import PY3
from pkg_resources import parse_version
import shutil
import json

try:
    import configobj
    if (PY3 and sys.version_info.minor >= 7 and
            parse_version(configobj.__version__) < parse_version('5.1.0')):
        raise ImportError('Installed configobj does not support Python 3.7+')
    _haveConfigobj = True
except ImportError:
    _haveConfigobj = False


if _haveConfigobj:  # Use the "global" installation.
    from configobj import ConfigObj
    try:
        from configobj import validate
    except ImportError:  # Older versions of configobj
        import validate
else:  # Use our contrib package if configobj is not installed or too old.
    from psychopy.contrib import configobj
    from psychopy.contrib.configobj import ConfigObj
    from psychopy.contrib.configobj import validate
join = os.path.join


class Preferences(object):
    """Users can alter preferences from the dialog box in the application,
    by editing their user preferences file (which is what the dialog box does)
    or, within a script, preferences can be controlled like this::

        import psychopy
        psychopy.prefs.hardware['audioLib'] = ['PTB', 'pyo','pygame']
        print(prefs)
        # prints the location of the user prefs file and all the current vals

    Use the instance of `prefs`, as above, rather than the `Preferences` class
    directly if you want to affect the script that's running.
    """

    def __init__(self):
        super(Preferences, self).__init__()
        self.userPrefsCfg = None  # the config object for the preferences
        self.prefsSpec = None  # specifications for the above
        # the config object for the app data (users don't need to see)
        self.appDataCfg = None

        self.general = None
        self.coder = None
        self.builder = None
        self.connections = None
        self.paths = {}  # this will remain a dictionary
        self.keys = {}  # does not remain a dictionary

        self.getPaths()
        self.loadAll()
        # setting locale is now handled in psychopy.localization.init
        # as called upon import by the app

        if self.userPrefsCfg['app']['resetPrefs']:
            self.resetPrefs()

    def __str__(self):
        """pretty printing the current preferences"""
        strOut = "psychopy.prefs <%s>:\n" % (
            join(self.paths['userPrefsDir'], 'userPrefs.cfg'))
        for sectionName in ['general', 'coder', 'builder', 'connections']:
            section = getattr(self, sectionName)
            for key, val in list(section.items()):
                strOut += "  prefs.%s['%s'] = %s\n" % (
                    sectionName, key, repr(val))
        return strOut

    def resetPrefs(self):
        """removes userPrefs.cfg, does not touch appData.cfg
        """
        userCfg = join(self.paths['userPrefsDir'], 'userPrefs.cfg')
        try:
            os.unlink(userCfg)
        except Exception:
            msg = "Could not remove prefs file '%s'; (try doing it manually?)"
            print(msg % userCfg)
        self.loadAll()  # reloads, now getting all from .spec

    def getPaths(self):
        # on mac __file__ might be a local path, so make it the full path
        thisFileAbsPath = os.path.abspath(__file__)
        prefSpecDir = os.path.split(thisFileAbsPath)[0]
        dirPsychoPy = os.path.split(prefSpecDir)[0]
        exePath = sys.executable

        # path to Resources (icons etc)
        dirApp = join(dirPsychoPy, 'app')
        if os.path.isdir(join(dirApp, 'Resources')):
            dirResources = join(dirApp, 'Resources')
        else:
            dirResources = dirApp

        self.paths['psychopy'] = dirPsychoPy
        self.paths['appDir'] = dirApp
        self.paths['appFile'] = join(dirApp, 'PsychoPy.py')
        self.paths['demos'] = join(dirPsychoPy, 'demos')
        self.paths['resources'] = dirResources
        self.paths['tests'] = join(dirPsychoPy, 'tests')
        # path to libs/frameworks
        if 'PsychoPy2.app/Contents' in exePath:
            self.paths['libs'] = exePath.replace("MacOS/python", "Frameworks")
        else:
            self.paths['libs'] = ''  # we don't know where else to look!

        if sys.platform == 'win32':
            self.paths['prefsSpecFile'] = join(prefSpecDir, 'Windows.spec')
            self.paths['userPrefsDir'] = join(os.environ['APPDATA'],
                                              'psychopy3')
        else:
            self.paths['prefsSpecFile'] = join(prefSpecDir,
                                               platform.system() + '.spec')
            self.paths['userPrefsDir'] = join(os.environ['HOME'],
                                              '.psychopy3')

        # Find / copy themes
        self.paths['themes'] = join(self.paths['userPrefsDir'], 'themes')
        baseThemes = join(self.paths['appDir'], 'themes')
        baseAppThemes = join(self.paths['appDir'], 'themes', 'app')
        # avoid silent fail-to-launch-app if bad permissions:

        try:
            os.makedirs(self.paths['userPrefsDir'])
        except OSError as err:
            if err.errno != errno.EEXIST:
                raise
        # Create themes folder in user space if not one already
        try:
            os.makedirs(self.paths['themes'])
        except OSError as err:
            if err.errno != errno.EEXIST:
                raise
        try:
            os.makedirs(join(self.paths['themes'], "app"))
        except OSError as err:
            if err.errno != errno.EEXIST:
                raise
        # Make sure all the base themes are present in user's folder
        #try:
        for file in os.listdir(baseThemes):
            if file.endswith('.json'):
                shutil.copyfile(
                    join(baseThemes, file),
                    join(self.paths['themes'], file)
                )
        for file in os.listdir(baseAppThemes):
            if file.endswith('.json'):
                shutil.copyfile(
                    join(baseAppThemes, file),
                    join(self.paths['themes'], "app", file)
                    )

    def loadAll(self):
        """Load the user prefs and the application data
        """
        self._validator = validate.Validator()

        # note: self.paths['userPrefsDir'] gets set in loadSitePrefs()
        self.paths['appDataFile'] = join(
            self.paths['userPrefsDir'], 'appData.cfg')
        self.paths['userPrefsFile'] = join(
            self.paths['userPrefsDir'], 'userPrefs.cfg')

        # If PsychoPy is tucked away by Py2exe in library.zip, the preferences
        # file cannot be found. This hack is an attempt to fix this.
        libzip = "\\library.zip\\psychopy\\preferences\\"
        if libzip in self.paths["prefsSpecFile"]:
            self.paths["prefsSpecFile"] = self.paths["prefsSpecFile"].replace(
                libzip, "\\resources\\")

        self.userPrefsCfg = self.loadUserPrefs()
        self.appDataCfg = self.loadAppData()
        self.validate()

        # simplify namespace
        self.general = self.userPrefsCfg['general']
        self.app = self.userPrefsCfg['app']
        self.coder = self.userPrefsCfg['coder']
        self.builder = self.userPrefsCfg['builder']
        self.hardware = self.userPrefsCfg['hardware']
        self.connections = self.userPrefsCfg['connections']
        self.appData = self.appDataCfg

        # keybindings:
        self.keys = self.userPrefsCfg['keyBindings']

    def loadUserPrefs(self):
        """load user prefs, if any; don't save to a file because doing so
        will break easy_install. Saving to files within the psychopy/ is
        fine, eg for key-bindings, but outside it (where user prefs will
        live) is not allowed by easy_install (security risk)
        """
        self.prefsSpec = ConfigObj(self.paths['prefsSpecFile'],
                                   encoding='UTF8', list_values=False)

        # check/create path for user prefs
        if not os.path.isdir(self.paths['userPrefsDir']):
            try:
                os.makedirs(self.paths['userPrefsDir'])
            except Exception:
                msg = ("Preferences.py failed to create folder %s. Settings"
                       " will be read-only")
                print(msg % self.paths['userPrefsDir'])
        # then get the configuration file
        cfg = ConfigObj(self.paths['userPrefsFile'],
                        encoding='UTF8', configspec=self.prefsSpec)
        # cfg.validate(self._validator, copy=False)  # merge then validate
        # don't cfg.write(), see explanation above
        return cfg

    def saveUserPrefs(self):
        """Validate and save the various setting to the appropriate files
        (or discard, in some cases)
        """
        self.validate()
        if not os.path.isdir(self.paths['userPrefsDir']):
            os.makedirs(self.paths['userPrefsDir'])
        self.userPrefsCfg.write()

    def loadAppData(self):
        # fetch appData too against a config spec
        appDataSpec = ConfigObj(join(self.paths['appDir'], 'appData.spec'),
                                encoding='UTF8', list_values=False)
        cfg = ConfigObj(self.paths['appDataFile'],
                        encoding='UTF8', configspec=appDataSpec)
        resultOfValidate = cfg.validate(self._validator,
                                        copy=True,
                                        preserve_errors=True)
        self.restoreBadPrefs(cfg, resultOfValidate)
        # force favComponent level values to be integers
        if 'favComponents' in cfg['builder']:
            for key in cfg['builder']['favComponents']:
                _compKey = cfg['builder']['favComponents'][key]
                cfg['builder']['favComponents'][key] = int(_compKey)
        return cfg

    def saveAppData(self):
        """Save the various setting to the appropriate files
        (or discard, in some cases)
        """
        # copy means all settings get saved:
        self.appDataCfg.validate(self._validator, copy=True)
        if not os.path.isdir(self.paths['userPrefsDir']):
            os.makedirs(self.paths['userPrefsDir'])
        self.appDataCfg.write()

    def validate(self):
        """Validate (user) preferences and reset invalid settings to defaults
        """
        result = self.userPrefsCfg.validate(self._validator, copy=True)
        self.restoreBadPrefs(self.userPrefsCfg, result)

    def restoreBadPrefs(self, cfg, result):
        """result = result of validate
        """
        if result == True:
            return
        vtor = validate.Validator()
        for sectionList, key, _ in configobj.flatten_errors(cfg, result):
            if key is not None:
                _secList = ', '.join(sectionList)
                val = cfg.configspec[_secList][key]
                cfg[_secList][key] = vtor.get_default_value(val)
            else:
                msg = "Section [%s] was missing in file '%s'"
                print(msg % (', '.join(sectionList), cfg.filename))

prefs = Preferences()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Part of the PsychoPy library
# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
# Distributed under the terms of the GNU General Public License (GPL).

from ._alerts import _activeAlertHandlers

class _BaseErrorHandler(object):
    """A base class for handling PsychoPy alerts and exceptions.
    """

    def __init__(self, alwaysReceive=True, autoFlush=True):
        self.errors = []
        self.alerts = []
        self.alwaysReceive = alwaysReceive
        self.autoFlush = autoFlush
        _activeAlertHandlers.append(self)

    def write(self, toWrite):
        """This is needed for any Python Exceptions, which assume the stderr
        is a file-like object. But we might well simply store the message for
        printing later.
        """
        self.errors.append(toWrite)
        if self.autoFlush:
            self.flush()

    def flush(self):
        """Print errors and alerts to console and clear errors.
        """
        for err in self.errors:
            print(err)
        self.errors = []

    def receiveAlert(self, alert):
        """
        Handles PsychoPy alerts (sent by _alerts.alert).
        This function should ONLY be called by _alerts.alert.

        Parameters:
        -----------
        alert: psychopy.alert._alert.AlertEntry object
            A data object containing alert information.
        """
        self.alerts.append(alert)

    def __del__(self):
        self.flush()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Part of the PsychoPy library
# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
# Distributed under the terms of the GNU General Public License (GPL).

from __future__ import absolute_import, print_function

from os import path
from psychopy.experiment.components import Param, getInitVals, _translate, BaseVisualComponent
from psychopy.visual import form

__author__ = 'Jon Peirce, David Bridges, Anthony Haffey'

# the absolute path to the folder containing this path
thisFolder = path.abspath(path.dirname(__file__))
iconFile = path.join(thisFolder, 'form.png')
tooltip = _translate('Form: a Psychopy survey tool')

# only use _localized values for label values, nothing functional:
_localized = {
    'Items': _translate('Items'),
    'Text Height': _translate('Text Height'),
    'Size': _translate('Size'),
    'Pos': _translate('Pos'),
    'Style': _translate('Styles'),
    'Item Padding': _translate('Item Padding'),
    'Data Format': _translate('Data Format'),
    'Randomize': _translate('Randomize')
    }
knownStyles = form.Form.knownStyles


class FormComponent(BaseVisualComponent):
    """A class for presenting a survey as a Builder component"""

    categories = ['Responses']
    targets = ['PsychoPy', 'PsychoJS']

    def __init__(self, exp, parentName,
                 name='form',
                 items='.csv',
                 textHeight=.03,
                 randomize=False,
                 size=(1, .7),
                 pos=(0, 0),
                 style=['dark'],
                 itemPadding=0.05,
                 startType='time (s)', startVal='0.0',
                 stopType='duration (s)', stopVal='',
                 startEstim='', durationEstim=''):

        super(FormComponent, self).__init__(
            exp, parentName, name=name,
            pos=pos, size=size,
            startType=startType, startVal=startVal,
            stopType=stopType, stopVal=stopVal,
            startEstim=startEstim, durationEstim=durationEstim)

        # these are defined by the BaseVisual but we don't want them
        del self.params['color']
        del self.params['colorSpace']
        del self.params['ori']
        del self.params['units']  # we only support height units right now

        self.type = 'Form'
        self.url = "http://www.psychopy.org/builder/components/"
        self.exp.requirePsychopyLibs(['visual', 'event', 'logging'])

        # params
        self.order = ['name',
                      'Items',
                      'Size', 'Pos',
                      'Data Format',
                      'Randomize',
                      ]

        # normal params:
        # = the usual as inherited from BaseComponent plus:

        self.params['Items'] = Param(
            items, valType='str', allowedTypes=[],
            updates='constant',
            hint=_translate("The csv filename containing the items for your survey."),
            label=_localized['Items'])

        self.params['Text Height'] = Param(
            textHeight, valType='code', allowedTypes=[],
            updates='constant',
            hint=_translate("The size of the item text for Form"),
            label=_localized['Text Height'],
            categ="Appearance")

        self.params['Randomize'] = Param(
            randomize, valType='bool', allowedTypes=[],
            updates='constant',
            hint=_translate("Do you want to randomize the order of your questions?"),
            label=_localized['Randomize'])

        self.params['Style'] = Param(
            style, valType='fixedList',
            updates='constant', allowedVals=knownStyles,
            hint=_translate(
                    "Styles determine the appearance of the form"),
            label=_localized['Style'],
            categ="Appearance")

        self.params['Item Padding'] = Param(
            itemPadding, valType='code', allowedTypes=[],
            updates='constant',
            hint=_translate("The padding or space between items."),
            label=_localized['Item Padding'],
            categ="Appearance")

        self.params['Data Format'] = Param(
            'rows', valType='str', allowedTypes=[],
            allowedVals=['columns', 'rows'],
            updates='constant',
            hint=_translate("Store item data by columns, or rows"),
            label=_localized['Data Format'])

    def writeInitCode(self, buff):
        inits = getInitVals(self.params)
        # build up an initialization string for Form():
        initStr = ("win.allowStencil = True\n"
                   "{name} = visual.Form(win=win, name='{name}',\n"
                   "    items={Items},\n"
                   "    textHeight={Text Height},\n"
                   "    randomize={Randomize},\n"
                   "    size={size},\n"
                   "    pos={pos},\n"
                   "    style={Style},\n"
                   "    itemPadding={Item Padding},"
                   ")\n".format(**inits))
        buff.writeIndented(initStr)

    def writeInitCodeJS(self, buff):
        inits = getInitVals(self.params)
        # build up an initialization string for Form():
        initStr = ("{name} = new visual.Form({{\n"
                   "  win : psychoJS.window, name:'{name}',\n"
                   "  items : {Items},\n"
                   "  textHeight : {Text Height},\n"
                   "  randomize : {Randomize},\n"
                   "  size : {size},\n"
                   "  pos : {pos},\n"
                   "  style : {Style},\n"
                   "  itemPadding : {Item Padding}\n"
                   "}});\n".format(**inits))
        buff.writeIndentedLines(initStr)

    def writeRoutineEndCode(self, buff):
        # save data, according to row/col format
        buff.writeIndented("{name}.addDataToExp(thisExp, {Data Format})\n"
                           .format(**self.params))
        buff.writeIndented("{name}.autodraw = False\n"
                           .format(**self.params))

    def writeRoutineEndCodeJS(self, buff):
        # save data, according to row/col format
        buff.writeIndented("{name}.addDataToExp(psychoJS.experiment, {Data Format});\n"
                           .format(**self.params))



from psychopy import visual, event, core
from psychopy.hardware import keyboard


kb = keyboard.Keyboard()

win = visual.Window()
msg = visual.TextStim(win, 'Using WaitKeys()\n\nPress a key!', wrapWidth=1.5)
msg.setAutoDraw(True)
instruct = visual.TextStim(win, pos=(0,-0.8),
    text='Press Esc to move on (to test getKeys)', wrapWidth=1.5)
instruct.setAutoDraw(True)
win.flip()

timer = core.Clock()

continuing = True
while continuing:
    timer.reset()
    kb.clock.reset()
    
    pygKey = event.waitKeys()[0]
    pygRT = timer.getTime()
    core.wait(0.001)  # make sure event was processed by ptb too
    ptbKey = kb.getKeys(waitRelease=False)[0]
    
    txt = 'got key:\n\n'
    txt += ('waitKeys: {},\n    RT={}\n\n'
            .format(pygKey, pygRT))
    txt += ('new(PTB): {} ({})\n    RT={}\n\n'
            .format(ptbKey.name, ptbKey.code, ptbKey.rt))
    txt += ('new was {:.3f}ms faster'
            .format((pygRT-ptbKey.rt)*1000))
    msg.text = txt

    if pygKey == 'escape':
        continuing = False

    win.flip()

msg.text = 'Using getKeys (worse timing)\n\nPress a key!'
instruct.text='Press Esc to finish'
continuing = True
timer.reset()
kb.clock.reset()
while continuing:
    
    pygKey = event.getKeys()
    pygRT = timer.getTime()
    if pygKey:
        pygKey = pygKey[0] # key not a list of them
        core.wait(0.001)  # make sure event was processed by ptb too
        # wait for the key to be raised
        ptbKey = kb.getKeys(waitRelease=False)[0]
        
        txt = 'got key:\n\n'
        txt += ('waitKeys: {},\n    RT={}\n\n'
                .format(pygKey, pygRT))
        txt += ('new(PTB): {} ({})\n    RT={}\n\n'
                .format(ptbKey.name, ptbKey.code, ptbKey.rt))
        txt += ('new was {:.3f}ms faster'
                .format((pygRT-ptbKey.rt)*1000))
        msg.text = txt

        if pygKey == 'escape':
            continuing = False
        #after this key press reset for next one
        win.callOnFlip(timer.reset)
        win.callOnFlip(kb.clock.reset)

    win.flip()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Part of the PsychoPy library
Copyright (C) 2015 Jonathan Peirce
Distributed under the terms of the GNU General Public License (GPL).
"""

from __future__ import absolute_import, print_function
from builtins import super  # provides Py3-style super() using python-future

from os import path
from psychopy.experiment.components import BaseComponent, Param, _translate
import numpy as np

# the absolute path to the folder containing this path
thisFolder = path.abspath(path.dirname(__file__))
iconFile = path.join(thisFolder, 'variable.png')
tooltip = _translate('Variable: create a new variable')

# only use _localized values for label values, nothing functional:
_localized = {'name': _translate('Name'),
              'startExpValue': _translate('Experiment start value'),
              'startRoutineValue': _translate('Routine start value'),
              'startFrameValue': _translate('Frame start value'),
              'saveStartExp': _translate('Save exp start value'),
              'saveStartRoutine': _translate('Save routine start value'),
              'saveFrameValue': _translate('Save frame value'),
              'saveEndRoutine': _translate('Save routine end value'),
              'saveEndExp': _translate('Save exp end value')}


class VariableComponent(BaseComponent):
    """An class for creating variables in builder."""

    def __init__(self, exp, parentName,
                 name='var1', startExpValue = '',
                 startRoutineValue='',
                 startFrameValue=''):

        super().__init__(exp, parentName, name)

        categories = ['Custom']
        self.type = 'Variable'
        self.url = "http://www.psychopy.org/builder/components/variable.html"
        self.order += ['startExpValue', 'saveStartExp', 'startRoutineValue', 'saveStartRoutine', 'startFrameValue',
                       'saveFrameValue', 'saveEndRoutine', 'saveEndExp']

        # set parameters
        hnt = _translate("The start value. A variable can be set to any value.")
        self.params['startExpValue'] = Param(
            startExpValue, valType='code', allowedTypes=[], updates='constant',
            hint=hnt,
            label=_localized['startExpValue'])
        hnt = _translate("Set the value for the beginning of each routine.")
        self.params['startRoutineValue'] = Param(
            startRoutineValue, valType='code', allowedTypes=[], updates='constant',
            hint=hnt,
            label=_localized['startRoutineValue'])
        hnt = _translate("Set the value for the beginning of every screen refresh.")
        self.params['startFrameValue'] = Param(
            startFrameValue, valType='code', allowedTypes=[],
            hint=hnt,
            label=_localized['startFrameValue'])
        # Save options
        hnt = _translate('Save the experiment start value in data file.')
        self.params['saveStartExp'] = Param(
            False, valType='bool',
            updates='constant',
            hint=hnt,
            label=_localized['saveStartExp'],
            categ='Save')
        hnt = _translate('Save the experiment end value in data file.')
        self.params['saveEndExp'] = Param(
            False, valType='bool',
            updates='constant',
            hint=hnt,
            label=_localized['saveEndExp'],
            categ='Save')
        hnt = _translate('Save the routine start value in data file.')
        self.params['saveStartRoutine'] = Param(
            False, valType='bool',
            updates='constant',
            hint=hnt,
            label=_localized['saveStartRoutine'],
            categ='Save')
        hnt = _translate('Save the routine end value in data file.')
        self.params['saveEndRoutine'] = Param(
            True, valType='bool',
            updates='constant',
            hint=hnt,
            label=_localized['saveEndRoutine'],
            categ='Save')
        hnt = _translate('Save choice of frame value in data file.')
        self.params['saveFrameValue'] = Param(
            'never', valType='str',
            allowedVals=['first', 'last', 'all', 'never'],
            updates='constant',
            hint=hnt,
            label=_localized['saveFrameValue'],
            categ='Save')

    def writeInitCode(self, buff):
        """Write variable initialisation code."""
        code = ("# Set experiment start values for variable component %(name)s\n")
        if self.params['startExpValue'] == '':
            code += ("%(name)s = ''\n")
        else:
            code += ("%(name)s = %(startExpValue)s\n")
        # Create variable container
        code += ("%(name)sContainer = []\n")
        buff.writeIndented(code % self.params)
    #
    def writeRoutineStartCode(self, buff):
        """Write the code that will be called at the start of the routine."""
        if not self.params['startRoutineValue'] == '':
            code = ("%(name)s = %(startRoutineValue)s  # Set routine start values for %(name)s\n")
            if self.params['saveStartRoutine'] == True:
                code += ("thisExp.addData('%(name)s.routineStartVal', %(name)s)  # Save exp start value\n")
            buff.writeIndentedLines(code % self.params)

    def writeFrameCode(self, buff):
        """Write the code that will be called at the start of the frame."""
        if not self.params['startFrameValue'] == '':
            basestring = (str, bytes)
            # Create dict for hold start and end types and converting them from types to variables
            timeTypeDict = {'time (s)': 't', 'frame N': 'frameN', 'condition': self.params['startVal'].val,
                            'duration (s)': 't','duration (frames)': 'frameN'}
            # Useful values for string creation
            startType = timeTypeDict[self.params['startType'].val]
            endType = timeTypeDict[self.params['stopType'].val]
            code = ''
            # Create default string
            frameCode = ("%(name)s = %(startFrameValue)s  # Set frame start values for %(name)s\n" % self.params)
            if not self.params['saveFrameValue'] == 'Never':
                frameCode += ("%(name)sContainer.append(%(name)s)  # Save frame values\n" % self.params)
            # Check for start or end values, and commence conditional timing string creation
            if self.params['startVal'].val or self.params['stopVal'].val:
                if self.params['startType'].val == 'time (s)':
                    # if startVal is an empty string then set to be 0.0
                    if (isinstance(self.params['startVal'].val, basestring) and
                            not self.params['startVal'].val.strip()):
                        self.params['startVal'].val = '0.0'

                # Begin string construction for start values
                if startType == 't':
                    code = (('if ' + startType + ' >= %(startVal)s') % self.params)
                elif startType == 'frameN':
                    code = (('if ' + startType + ' >= %(startVal)s') % self.params)
                elif self.params['startType'].val == 'condition':
                    code = ('if bool(%(startVal)s)' % self.params)

                # Begin string construction for end values
                if not self.params['stopVal'].val:
                    code += (':\n' % self.params)
                # Duration types must be calculated
                elif u'duration' in self.params['stopType'].val:
                    if 'frame' in self.params['startType'].val and 'frame' in self.params['stopType'].val \
                            or '(s)' in self.params['startType'].val and '(s)' in self.params['stopType'].val:
                        endTime = str((float(self.params['startVal'].val) + float(self.params['stopVal'].val)))
                    else:  # do not add mismatching value types
                        endTime = self.params['stopVal'].val
                    code += (' and ' + endType + ' <= ' + endTime + ':\n' % (self.params))
                elif endType == 't' :
                    code += (' and ' + endType + ' <= %(stopVal)s:\n' % (self.params))
                elif endType == 'frameN' :
                    code += (' and ' + endType + ' <= %(stopVal)s:\n' % (self.params))
                elif self.params['stopType'].val == 'condition':
                    code += (' and bool(%(stopVal)s):\n' % self.params)
                code += ''.join(['    ' + lines + '\n' for lines in frameCode.splitlines()])
            else:
                code = frameCode
            buff.writeIndentedLines(code)

    def writeRoutineEndCode(self, buff):
        """Write the code that will be called at the end of the routine."""
        code = ''
        if self.params['saveStartExp'] == True and not self.params['startExpValue'] == '':
            code = ("thisExp.addData('%(name)s.expStartVal', %(startExpValue)s)  # Save exp start value\n")
        if self.params['saveEndRoutine'] == True and not self.params['startRoutineValue'] == '':
            code += ("thisExp.addData('%(name)s.routineEndVal', %(name)s)  # Save end routine value\n")
        if not self.params['startFrameValue'] == '':
            if self.params['saveFrameValue'] == 'last':
                code += ("thisExp.addData('%(name)s.frameEndVal', %(name)sContainer[-1])  # Save end frame value\n")
            elif self.params['saveFrameValue'] == 'first':
                code += ("thisExp.addData('%(name)s.frameStartVal', %(name)sContainer[0])  # Save start frame value\n")
            elif self.params['saveFrameValue'] == 'all':
                code += ("thisExp.addData('%(name)s.allFrameVal', %(name)sContainer)  # Save all frame value\n")
        buff.writeIndentedLines(code % self.params)

    def writeExperimentEndCode(self, buff):
        """Write the code that will be called at the end of the experiment."""
        code=''
        writeData = []
        # For saveEndExp, check whether any values were initiated.
        for vals in ['startExpValue', 'startRoutineValue', 'startFrameValue']:
            if not self.params[vals] == '':
                writeData.append(True)
        # Write values to file if requested, and if any variables defined
        if self.params['saveEndExp'] == True and np.any(writeData):
            code = ("thisExp.addData('%(name)s.endExpVal', %(name)s)  # Save end experiment value\n")
        buff.writeIndentedLines(code % self.params)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Demo of text rendering in pyglet, including:
- how to specify fonts
- unicode
- rotating text
- mirror-image
- bidirectional and reshaped Arabic/Farsi text
"""

from __future__ import division

from psychopy import visual, core, event

# Create a window to draw in
win = visual.Window((800.0, 800.0), allowGUI=True, winType='pyglet',
            monitor='testMonitor', units ='deg', screen=0)
win.recordFrameIntervals = True

# Choose some fonts. If a list is provided, the first font found will be used.
fancy = ['Monotype Corsiva', 'Palace Script MT', 'Edwardian Script ITC']
sans = ['Gill Sans MT', 'Arial', 'Helvetica', 'Verdana']
serif = ['Times', 'Times New Roman']
comic = 'Comic Sans MS'  # the short name won't work

rs = visual.RatingScale(win, low=0, high=1, precision=100, size=3, pos=(0,-.4),
                       labels=[' ', ' '], scale=' ',
                       marker='glow', markerStart=0.7, markerColor='darkBlue', autoLog=False)
rs.draw()
# Initialize some stimuli.
# Note that in Python 3 we no longer need to create special unicode strings
# with a u'' prefix, as all strings are unicode. For the time being, we 
# retain the prefix in this demo, for backwards compatibility for people 
# running PsychoPy under Python 2.7
fpsText = visual.TextStim(win)
rotating = visual.TextStim(win, text="Fonts \nrotate", pos=(0, 0),  # and can have line breaks
    color=[-1.0, -1, 1],
    units='deg', 
    ori=0, height = 1.0,
    font=comic)
unicodeStuff = visual.TextStim(win,
    # you can find the unicode character value by searching online
    text = u"unicode (eg \u03A8 \u040A \u03A3)",
    color='black',
    font=serif, pos=(0, 3),
    height = 1)
psychopyTxt = visual.TextStim(win, color='#FFFFFF',
    text = u"PsychoPy \u00A9Jon Peirce",
    units='norm', height=0.05,
    pos=[0.95, 0.95], 
    alignText='right', anchorHoriz='right',
    font=fancy)
longSentence = visual.TextStim(win,
    text = u"Very long sentences can wrap", wrapWidth=0.4,
    units='norm', height=0.05, color='DarkSlateBlue',
    pos=[0.95, -0.95], 
    alignText='left', anchorHoriz='right', # right of the window, left of the box!
    anchorVert='bottom')
mirror = visual.TextStim(win, text="mirror mirror",
    units='norm', height=0.12, color='Silver',
    pos=[0, -0.5],
    flipHoriz=True)

# By default, right-to-left languages like Hebrew are often shown in
# reversed order. Additionally, Arabic-based text by default is shown
# with characters in their isolated form, rather than flowing correctly
# into their neighbours. We can use the invisible \u200E left-to-right
# control character to resolve ambiguous transitions between text 
# directions (for example, to determine in which directional run a 
# punctuation character belongs).
raw_Farsi = visual.TextStim(win,
    text = u'Raw Farsi text: \n \u200E خوش آمدید 1999',
    units = 'norm', height = 0.06, color = 'DarkRed',
    pos = (-0.9, 0.8), font = 'Arial',
    wrapWidth = 1.0, 
    alignText='left', anchorHoriz='left',
    languageStyle = 'LTR') # left-to-right
# We correct these issues by setting setting the languageStyle to be
# 'bidirectional' (sufficient for Hebrew, for example) or 'Arabic'
# (which additionally does the reshaping of individual characters
# needed for languages based on the Arabic alphabet):
corrected_Farsi = visual.TextStim(win,
    text = u'Reshaped & bidirectional: \n \u200E خوش آمدید 1999',
    units = 'norm', height = 0.06, color = 'DarkRed',
    pos = (-0.9, 0.6), font = 'Arial',
    wrapWidth = 1.0, 
    alignText='left', anchorHoriz='left',
    languageStyle = 'Arabic') # RTL + reshaped
# Please give the developers feedback if there are display issues in
# other languages that you are familiar with.

trialClock = core.Clock()
t = lastFPSupdate = 0

# Continues the loop until any key is pressed
while not event.getKeys():
    t = trialClock.getTime()
    mirror.draw()
    rotating.ori += 1
    rotating.draw()
    unicodeStuff.draw()
    longSentence.draw()
    raw_Farsi.draw()
    corrected_Farsi.draw()
#
#    # update the fps text every second
    if t - lastFPSupdate > 1:
        fpsText.text = "%i fps" % win.fps()
        lastFPSupdate += 1
    fpsText.draw()
    psychopyTxt.draw()

    win.flip()

win.close()
core.quit()

# The contents of this file are in the public domain.
